#include <stdlib.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdint.h>
#include <avr/wdt.h>


#include "comm.h" 

//declare the global variables
static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar,NULL,_FDEV_SETUP_WRITE);
volatile unsigned char Configuration[33];//this holds the cc1020 register settings
unsigned char defaultconfig[] = {0x01,0x01,0xFF,0x8A,0x3A,0x15,0x77,0x3F,0x3A,0x15,0x77,0x3F,0x44,0x50,0x9A,0xDC,0x3F,0xE5,0x77,0x38,0x38,0x90,0x78,0x47,0x14,0x22,0xBE,0x34,0x00,0x00,0x00,0x00,0x80};
char buffer[BUFFER_SIZE];
unsigned char buffer_top=0,buffer_bottom=0;
volatile unsigned char ireg,qreg,regdump=0;

//this is to fix the 15ms wdt timer reset loop issue
uint8_t mcusr_mirror __attribute__ ((section (".noinit")));
void get_mcusr(void) \			
__attribute__((naked)) \
__attribute__((section(".init3")));
void get_mcusr(void)
{
	mcusr_mirror = MCUSR;
	MCUSR = 0;
	wdt_disable();
}

int uart_putchar(char c, FILE *stream)
{
	if (c == '\n')
		uart_putchar('\r', stream);
	loop_until_bit_is_set(UCSR0A, UDRE0);
	UDR0 = c;
	return 0;
}

char get_from_buffer(void)			//gets a chr from the buffer and also checks for incoming data
{
	char c;
	if((buffer_bottom==buffer_top)||regdump)//there is nothing in the buffer or unprocessed data
	{	
		for(;!regdump;);		//if necessary, loop until there is some data avaliable
		add_to_buffer(ireg);
		add_to_buffer(qreg);
		regdump=0;			//add this data to the buffer
	}
	c=buffer[buffer_bottom];
	buffer_bottom++;
	if(buffer_bottom>=BUFFER_SIZE)		//loop back to the beginning
			buffer_bottom=0;
	return c;
}

void add_to_buffer(char c)			//adds a character to the buffer
{
	buffer[buffer_top]=c;
	buffer_top++;
	if(buffer_top>=BUFFER_SIZE)		//loop back to the beginning
		buffer_top=0;
}

ISR(INT0_vect)					//INT0 interrupt on mega xx8 series
{
	SS_LOW;
	SPDR = (0x49<<1);
	loop_until_bit_is_set(SPSR,SPIF);
	ireg = SPDR;				//have to read this to clear the flag
	SPDR = 0x00;				//dummy write
	loop_until_bit_is_set(SPSR,SPIF);
	ireg = SPDR;
	SS_HIGH;
//	ireg = ReadFromCC1020Register(0x49);	//dump the I and Q registers
	SS_LOW;
	SPDR = (0x4A<<1);
	loop_until_bit_is_set(SPSR,SPIF);
	qreg = SPDR;				//have to read this to clear the flag
	SPDR = 0x00;				//dummy write
	loop_until_bit_is_set(SPSR,SPIF);
	qreg = SPDR;
	SS_HIGH;
//	qreg = ReadFromCC1020Register(0x4A);
	regdump=1;				//so we know the registers have been dumped
}	
		
int main(void)
{
	UCSR0A = (1<<U2X0);			//double speed usart operation
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);		//mega xx8 registers - enable tx and rx, no interrupts
	UCSR0C = (1<<UCSZ01)|(1<<UCSZ00);	//setup 8N1
	UBRR0L = 0x01;
	UBRR0H = 0x00;				//end of UART setup - this will give us a 460800 baud uart
	PORTC=0x00;
	DDRC=0x00;
	DDRB = (1<<1)|(1<<2)|(1<<3)|(1<<5);	//setup portb as spi comms + led
	SPCR = (1<<SPE)|(1<<MSTR);		//setup SPI as master
	SPSR = (1<<SPI2X);			//setup SPI double speed ie FOSC/2
	EIMSK |= (1<<INT0);  			//Enable INT0
	EICRA |= ((1<<ISC01)|(1<<ISC00));	//Trigger on rising edge of INT0
	//TCCR0A = (1<<WGM01);			//setup timer0 to overflow on compare A match
	//TCCR0B = (1<<CS01);			//set the FOSC as timer0/8 sourcef
	//OCR0A = (unsigned char)58;		//overflow on 48 giving 16Ksps
	stdout = &mystdout;
	unsigned char n=0,old_l=0,l=0;
	signed long plldivider=0x003A1577;	//we setup in Rx mode operating at 434.075mhz with 9.6khz filter
	LED_ON;
	_delay_ms(100);
	wdt_enable(WDTO_1S);
	memcpy(Configuration, defaultconfig, sizeof(defaultconfig));//sets up the default configuration
	if(mcusr_mirror & ((1<<BORF)|(1<<PORF)))//we had a power on reset, setup the cc1020
	{
		LED_OFF;
		if(InitializeCC1020(Configuration)==3)//setup the cc1020, this will also enter the default settings into Configuration
			LED_ON;				//led on if both calibrations were okay
		SetupCC1020PD();			//cc1020 is now powered down
		WakeUpCC1020ToRX(Configuration);
		if(SetupCC1020RX(Configuration)==LOCK_NOK)//setup to Rx, led off if not successful
			LED_OFF;
		_delay_ms(200);
	}
	sei();
	while(1)
	{
		add_to_buffer(SYNC_CHR);		//the sync character sync,i,q,i,q,i,q,i,q,i,q,sync etc
		l = get_from_buffer();
		loop_until_bit_is_set(UCSR0A, UDRE0);	//wait for the last character we loaded to be sent
		UDR0 = l;	
		for(n=0;n<5;n++)
		{
			l = get_from_buffer();
			loop_until_bit_is_set(UCSR0A, UDRE0);	//wait for the last character we loaded to be sent
			UDR0 = l;
			l = get_from_buffer();
			loop_until_bit_is_set(UCSR0A, UDRE0);	//wait for the last character we loaded to be sent
			UDR0 = l;
			if(l!=old_l)		//if the data isnt updating the watchdog reset will be triggered
				wdt_reset();
			old_l=l;
		}
		//we now check to see if there has been any incoming data
		if(UCSR0A & (1<<RXC0))
		{
			cli();
			switch(UDR0)
			{
				case 'r'://'r' triggers a reset of the hardware
					while(1)
					;//make the watchdog timer reset the micro so we can run the bootloader
				case 't'://'t' tunes us up or down in frequency
					while(!(UCSR0A & (1<<RXC0)))//wait for a new byte to come through
					;
					//SetupCC1020PD();
					//ResetCC1020();
					plldivider+=((signed char)UDR0);//the last bit of the frequency word is the dither bit - leave it unchanged
					Configuration[4]=(unsigned char)(plldivider>>16);
					Configuration[5]=(unsigned char)(plldivider>>8);
					Configuration[6]=(unsigned char)(plldivider&0x000000FF);
					/*ConfigureCC1020(Configuration);	//write these settings to the cc1020
					WakeUpCC1020ToRX(Configuration);
					if(!CalibrateCC1020())		//recalibrate, led off if not sucessful
						LED_OFF;
					if(SetupCC1020RX(Configuration)==LOCK_NOK)//setup to Rx, led off if not successful*/
					LED_OFF;
					if(InitializeCC1020(Configuration)==3)//setup the cc1020
						LED_ON;				//led on if both calibrations were okay
					SetupCC1020PD();			//cc1020 is now powered down
					WakeUpCC1020ToRX(Configuration);
					if(SetupCC1020RX(Configuration)==LOCK_NOK)//setup to Rx, led off if not successful
						LED_OFF;
					break;
				case 's'://'s' can be used to syncronise so we dont mix up I and Q samples - flush the input buffer then IQIQIQ etc
					LED_OFF;
					_delay_ms(100);
					LED_ON;
				default:
					;	
			}
			sei();
		}
	}
}



