#ifndef __CC1020_H
#define __CC1020_H

#include <stdlib.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdint.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <string.h>
#include "CC1020settings.h"
#include "../comm.h"

/**
 *  @defgroup CC1020 Chipcon CC1020 Configuration Library 
 *
 * The CC1020 library is an AVR-GCC Library which handles the configuration of the Chipcon CC1020.  
 * This includes routines for reading and writing to configuration registers.
 *
 * @{
 */


//prototypes

// CC1020 Register Definitions

/** Define the CC1020's MAIN register as address 0x00 */
#define CC1020_MAIN            0x00

/** Define the CC1020's INTERFACE register as address 0x01 */
#define CC1020_INTERFACE       0x01

/** Define the CC1020's RESET register as address 0x02 */
#define CC1020_RESET           0x02

/** Define the CC1020's SEQUENCING register as address 0x03 */
#define CC1020_SEQUENCING      0x03

/** Define the CC1020's FREQ_2A register as address 0x04 */
#define CC1020_FREQ_2A         0x04

/** Define the CC1020's FREQ_1A register as address 0x05 */
#define CC1020_FREQ_1A         0x05

/** Define the CC1020's FREQ_0A register as address 0x06 */
#define CC1020_FREQ_0A         0x06

/** Define the CC1020's CLOCK_A register as address 0x07 */
#define CC1020_CLOCK_A         0x07

/** Define the CC1020's FREQ_2B register as address 0x08 */
#define CC1020_FREQ_2B         0x08

/** Define the CC1020's FREQ_1B register as address 0x09 */
#define CC1020_FREQ_1B         0x09

/** Define the CC1020's FREQ_0B register as address 0x0A */
#define CC1020_FREQ_0B         0x0A

/** Define the CC1020's CLOCK_B register as address 0x0B */
#define CC1020_CLOCK_B         0x0B

/** Define the CC1020's VCO register as address 0x0C */
#define CC1020_VCO             0x0C

/** Define the CC1020's MODEM register as address 0x0D */
#define CC1020_MODEM           0x0D

/** Define the CC1020's DEVIATION register as address 0x0E */
#define CC1020_DEVIATION       0x0E

/** Define the CC1020's AFC_CONTROL register as address 0x0F */
#define CC1020_AFC_CONTROL     0x0F

/** Define the CC1020's FILTER register as address 0x10 */
#define CC1020_FILTER          0x10

/** Define the CC1020's VGA1 register as address 0x11 */
#define CC1020_VGA1            0x11

/** Define the CC1020's VGA2 register as address 0x12 */
#define CC1020_VGA2            0x12

/** Define the CC1020's VGA3 register as address 0x13 */
#define CC1020_VGA3            0x13

/** Define the CC1020's VGA4 register as address 0x14 */
#define CC1020_VGA4            0x14

/** Define the CC1020's LOCK register as address 0x15 */
#define CC1020_LOCK            0x15

/** Define the CC1020's FRONTEND register as address 0x16 */
#define CC1020_FRONTEND        0x16

/** Define the CC1020's ANALOG register as address 0x17 */
#define CC1020_ANALOG          0x17

/** Define the CC1020's BUFF_SWING register as address 0x18 */
#define CC1020_BUFF_SWING      0x18

/** Define the CC1020's BUFF_CURRENT register as address 0x19 */
#define CC1020_BUFF_CURRENT    0x19

/** Define the CC1020's PLL_BW register as address 0x1A */
#define CC1020_PLL_BW          0x1A

/** Define the CC1020's CALIBRATE register as address 0x1B */
#define CC1020_CALIBRATE       0x1B

/** Define the CC1020's PA_POWER register as address 0x1C */
#define CC1020_PA_POWER        0x1C

/** Define the CC1020's MATCH register as address 0x1D */
#define CC1020_MATCH           0x1D

/** Define the CC1020's PHASE_COMP register as address 0x1E */
#define CC1020_PHASE_COMP      0x1E

/** Define the CC1020's GAIN_COMP register as address 0x1F */
#define CC1020_GAIN_COMP       0x1F

/** Define the CC1020's POWERDOWN register as address 0x20 */
#define CC1020_POWERDOWN       0x20

/** Define the CC1020's TEST1 register as address 0x21 */
#define CC1020_TEST1           0x21

/** Define the CC1020's TEST2 register as address 0x22 */
#define CC1020_TEST2           0x22

/** Define the CC1020's TEST3 register as address 0x23 */
#define CC1020_TEST3           0x23

/** Define the CC1020's TEST4 register as address 0x24 */
#define CC1020_TEST4           0x24

/** Define the CC1020's TEST5 register as address 0x25 */
#define CC1020_TEST5           0x25

/** Define the CC1020's TEST6 register as address 0x26 */
#define CC1020_TEST6           0x26

/** Define the CC1020's TEST7 register as address 0x27 */
#define CC1020_TEST7           0x27

/** Define the CC1020's STATUS register as address 0x40 */
#define CC1020_STATUS          0x40

/** Define the CC1020's RESET_DONE register as address 0x41 */
#define CC1020_RESET_DONE      0x41

/** Define the CC1020's RSS register as address 0x42 */
#define CC1020_RSS             0x42

/** Define the CC1020's AFC register as address 0x43 */
#define CC1020_AFC             0x43

/** Define the CC1020's GAUSS_FILTER register as address 0x44 */
#define CC1020_GAUSS_FILTER    0x44

/** Define the CC1020's STATUS1 register as address 0x45 */
#define CC1020_STATUS1         0x45

/** Define the CC1020's STATUS2 register as address 0x46 */
#define CC1020_STATUS2         0x46

/** Define the CC1020's STATUS3 register as address 0x47 */
#define CC1020_STATUS3         0x47

/** Define the CC1020's STATUS4 register as address 0x48 */
#define CC1020_STATUS4         0x48

/** Define the CC1020's STATUS5 register as address 0x49 */
#define CC1020_STATUS5         0x49

/** Define the CC1020's STATUS6 register as address 0x4A */
#define CC1020_STATUS6         0x4A

/** Define the CC1020's STATUS7 register as address 0x4B */
#define CC1020_STATUS7         0x4B

// Set up some boolean conditions

/** A FALSE boolean condition. */
#define FALSE 0x00;

/** A TRUE boolean condition. */
#define TRUE 0x01;


// PIC Pin Assignment

/** Define PSEL as port B, bit 2 on the processor. */
#define PSEL		PORTBbits.RB2


// Timout Definitions

/** Timeout value to wait for Calibration to complete. */
#define CAL_TIMEOUT    0x7FFE

/** Timeout value while waiting for frequency lock. */
#define LOCK_TIMEOUT   0x7FFE

/** Timeout value to wait for Reset to complete. */
#define RESET_TIMEOUT  0x7FFE



/**
 * Lock Status of the CC1020.
 * Defines three states which the CC1020 may exhibit during calibration or receive.
 */
enum lockStatus {
	 LOCK_OK, /**< Frequency Lock is ok */
	 LOCK_RECAL_OK, /**< Frequency Lock after recalibration is ok */
	 LOCK_NOK /**< Frequency Lock is not ok */
	};


char InitializeCC1020(unsigned char* Configuration);
void ConfigureCC1020(unsigned char* Configuration);
void WriteToCC1020Register(char addr, char data);
char ReadFromCC1020Register(char addr);
void ResetCC1020(void);
char CalibrateCC1020(void);
char SetupCC1020RX(unsigned char* Configuration);
char SetupCC1020TX(void);
void SetupCC1020PD(void);
void WakeUpCC1020ToRX(unsigned char* Configuration);
void WakeUpCC1020ToTX(void);

/** @} */

#endif

