/**
 *  @page cc1020registers Register Settings
 *
 * <p>The CC1020 is configured to be compatible with the G3RUH modems readily available.  Since the CC1020 was not intended to be
 * used with such a narrow bandwidth to data rate spacing we had to tell the CC1020 to transmit with a frequency separation of
 * 3kHz to get it to transmit with a 6kHz separation.  The following is the output from Chipcon's SmartRF Studio software's
 * "Print Settings" menu option.  The actual register settings were further "tweaked" for better receive response.</p>
 *
 *
 * <p>The following configuration is good for these System parameters:</p>
 * <pre>
 * X-tal frequency: 14.745600 MHz	Internal
 * X-tal accuracy: +/-  5 ppm
 * RF frequency A: 435.245000 MHz	Inactive	Rx
 * RF frequency B: 435.245000 MHz	Active  	Tx
 * Frequency separation: 5.900 kHz
 * Data rate: 9.600 kBaud
 * Data Format: NRZ		Accurate
 * RF output power: 10 dBm
 * Channel spacing: 25 kHz
 * Modulation: GFSK		Dithering enabled
 * Lock: Continuous
 * Carrier sense offset:   0 dBm	DCLK squelch disabled
 * Operator Mode: Tx
 * </pre>
 */
 
 /** Set MAIN Register Value to 0xC1 for transmit mode.*/
#define MAIN            0xC1

/** Set MAIN Register Value to 0x01 for receive mode.*/
#define RXMAIN			0x01

/** Set INTERFACE Register to 0x0F for all modes.*/
#define INTERFACE       0x0F

/** Set RESET Register Value to 0xFF for all modes.*/
#define RESET           0xFF

/** Set SEQUENCING Register Value to 0x8F for all modes.*/
#define SEQUENCING      0x8F

/** Set FREQ_2A Register Value to 0x3A for 435.245 MHz.*/
#define FREQ_2A         0x3A

/** Set FREQ_1A Register Value to 0x3E for 435.245 MHz.*/
#define FREQ_1A         0x3E

/** Set FREQ_0A Register Value to 0x01 for 435.245 MHz.*/
#define FREQ_0A         0x01

/** Set CLOCK_A Register Value to 0x39 for 435.245 MHz and 9600 baud.*/
#define CLOCK_A         0x39

/** Set FREQ_2B Register Value to 0x3A for 435.245 MHz.*/
#define FREQ_2B         0x3A

/** Set FREQ_1B Register Value to 0x48 for 435.245 MHz.*/
#define FREQ_1B         0x48

/** Set FREQ_0B Register Value to 0xAD for 435.245 MHz.*/
#define FREQ_0B         0xAD

/** Set CLOCK_B Register Value to 0x39 for 435.245 MHz and 9600 baud.*/
#define CLOCK_B         0x39

/** Set VCO Register Value to 0x44 for all modes.*/
#define VCO             0x44

/** Set MODEM Register Value for 9600 baud.*/
#define MODEM           0x50

/** Set DEVIATION Register Value to 0x8D for 2.950 kHz frequency separation.*/
#define DEVIATION    	0x9D

/** Set TXDEVIATION Register Value to 0x8D for 2.950 kHz frequency separation.*/
#define TXDEVIATION     0x9D

/** Set RXDEVIATION Register Value to 0x9D for 2.950 kHz frequency separation.*/
#define RXDEVIATION     0x9D

/** Set AFC_CONTROL Register Value to 0xC1 for proper reception.  RFStudio says this should be 0xC3, 0xC1 seems to work better.*/
#define AFC_CONTROL     0xC1

/** Set FILTER Register to 0x2F for a 25 kHz wide channel spacing.  0x2B was tried for a wider spacing but 0x2F seems better */
#define FILTER          0x2F

/** Set VGA1 Register Value to 0x61 for all modes.*/
#define VGA1            0x61

/** Set VGA2 Register Value to 0x55 for all modes.*/
#define VGA2            0x55

/** Set VGA3 Register Value to 0x2F for all modes.*/
#define VGA3            0x2F	// 2E for RX

/** Set VGA4 Register Value to 0x2D for all modes.*/
#define VGA4            0x2D	// 29 for RX

/** Set LOCK Register Value to 0x20 for all modes.*/
#define LOCK            0x20	//lock pin is output

/** Set FRONTEND Register Value to 0x78 for all modes.*/
#define FRONTEND        0x78

/** Set ANALOG Register Value to 0x47 for all modes.*/
#define ANALOG          0x47

/** Set RXANALOG Register Value to 0x47 for all modes.*/
#define RXANALOG		0x47

/** Set TXANALOG Register Value to 0x47 for all modes.*/
#define TXANALOG		0x47

/** Set BUFF_SWING Register Value to 0x14 for all modes.*/
#define BUFF_SWING      0x14

/** Set BUFF_CURRENT Register Value to 0x22 for all modes.*/
#define BUFF_CURRENT    0x22

/** Set PLL_BW Register Value to 0xAE for all modes.*/
#define PLL_BW          0xAE

/** Set CALIBRATE Register Value to 0x34 for all modes.*/
#define CALIBRATE       0x34

/** Set PA_POWER Register Value to 0x0F for all modes.*/
#define PA_POWER        0x0F

/** Set PA_POWER Register Value to 0x60 for 0 dBm.*/
#define PA_POWER_0_DBM        0x60

/** Set PA_POWER Register Value to 0x80 for 5 dBm.*/
#define PA_POWER_5_DBM        0x80

/** Set PA_POWER Register Value to 0x05 for -10 dBm.*/
#define PA_POWER_n10_DBM        0x05

/** Set MATCH Register Value to 0x00 for all modes.*/
#define MATCH           0x00

/** Set PHASE_COMP Register Value to 0x00 for all modes.*/
#define PHASE_COMP      0x00

/** Set GAIN_COMP Register Value to 0x00 for all modes.*/
#define GAIN_COMP       0x00

/** Set POWERDOWN Register Value to 0x00 for all modes.*/
#define POWERDOWN       0x00

