#include "CC1020.h"

/**
 * Reads a byte from one of the CC1020 registers.
 * 
 * @pre SPI bus is open and configured
 *
 * @param addr The configuaration register address to read from.
 * 
 * @return The specified CC1020 register value.
 *
 * @remarks There is a strange timing issue with either the implementation of the
 * C18 SPI library or the CC1020's SPI implementation.  This is resolved by writing
 * directly to the registers rather than using the Microchip provided function calls.
 */

char ReadFromCC1020Register(char addr)
{
  char Value;

  PSEL=0;
  Value=SSPBUF;
  SSPBUF=(addr<<1)&0xFE; // Write address to CC1020, write bit is always 0



  // Wait until data is written 
  while (SSPSTATbits.BF==0);
  SSPCON1bits.SSPOV=0;

  // Switch direction 
  TRISC|=0x20; // Set up PDATAOUT as an input
  
  SSPBUF=0xFF; // Dummy write 
  
  while (SSPSTATbits.BF==0);
  Value=SSPBUF;

  TRISC&=~0x20; // Set PDATAOUT as an output 

  PSEL=1;

  return Value;
}

