#include "CC1020.h"

/**
 * This routine puts the CC1020 into RX mode.
 *
 * @pre CC1020 is not in receive mode.  If the CC1020 is coming out of Power Down mode, WakeupCC1020ToRx must be called first.
 *
 * @post CC1020 is in receive mode.
 *
 * @return Lock status.
 */
char SetupCC1020RX(void)
{
  volatile int TimeOutCounter;
  char lock_status;
	char dummy, addr;

  // Switch into RX, switch to freq. reg A
  WriteToCC1020Register(CC1020_MAIN,0x11);

  // Setup bias current adjustment
  WriteToCC1020Register(CC1020_ANALOG,RXANALOG);
  WriteToCC1020Register(CC1020_DEVIATION,RXDEVIATION);
  WriteToCC1020Register(CC1020_AFC_CONTROL,0xC3);
  WriteToCC1020Register(CC1020_VGA3,0x2E);
  WriteToCC1020Register(CC1020_VGA4,0x29);

  // Monitor LOCK
  for(TimeOutCounter=LOCK_TIMEOUT; ((ReadFromCC1020Register(CC1020_STATUS)&0x10)==0)&&(TimeOutCounter>0); TimeOutCounter--);

  // If PLL in lock
  if((ReadFromCC1020Register(CC1020_STATUS)&0x10)==0x10){
    // Indicate PLL in LOCK
    lock_status = LOCK_OK;
  // Else (PLL out of LOCK)
  }else{
    // If recalibration ok
    if(CalibrateCC1020()){
      // Indicate PLL in LOCK
      lock_status = LOCK_RECAL_OK;
    // Else (recalibration failed)
    }else{
      // Indicate PLL out of LOCK
      lock_status = LOCK_NOK;
    }
  }
	WriteToCC1020Register(CC1020_MAIN,RXMAIN);
  // Switch RX part of CC1020 on
  
  return lock_status;
}

