#!/usr/bin/python
"""
fldigi autotuner script
Released into the public domain by Adam Greig in 2010
Modified by Laurence Blaxter
"""
import sys
import xmlrpclib
import math
import time
import subprocess
import signal

if len(sys.argv) != 2:  # the program name and the one argument
 	# stop the program and print an error message
	sys.exit("Must provide a device(e.g. /dev/ttyUSB0)")

# Desired audio frequency (Hz)
demand_freq = 2500

#the start frequency of the hardware (approximate)
start_frequency = 434073000.0

# Loop time (s)
loop_time = 0.25

#the SDR PLL step size
pll_step = 225

# Connect to fldigi
s = xmlrpclib.ServerProxy("http://localhost:7362")

# Set the dial frequency correctly
s.main.set_frequency(start_frequency)

# Frequency is the actual frequency the SDR is on
frequency=start_frequency

#Fire up the c code - fix the python sigpipe issue
signal.signal(signal.SIGPIPE, signal.SIG_DFL)
bp = subprocess.Popen(['./pump', sys.argv[1]], shell=False, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

while 1:
	# Read current dial frequencie
	dial_freq = s.main.get_frequency()
	# Check the dial frequency to see if we need to retune the SDR - this allows manual retuning
	if abs(dial_freq-frequency)>=math.ceil(pll_step/2):
		if dial_freq > frequency:
			#retune by +225
			bp.stdin.write("1\n")
			frequency+=pll_step
		else:
			#return by -225
			bp.stdin.write("-1\n")
			frequency-=pll_step 	
	# Audio frequency on the waterfall
	audio_freq = s.modem.get_carrier()
	if abs(audio_freq - demand_freq) >= math.ceil(pll_step/2):
		if audio_freq > demand_freq:
			#retune by +225
			bp.stdin.write("1\n")
			frequency+=pll_step
			s.modem.set_carrier(audio_freq-pll_step)
		else:
			#retune by -225
			bp.stdin.write("-1\n")
			frequency-=pll_step
			s.modem.set_carrier(audio_freq+pll_step)
		#set the dial frequency correctly
		s.main.set_frequency(frequency)
	# Sleep a bit
	time.sleep(loop_time)
	bp.poll()
	if bp.returncode!=None:
		sys.exit("fldigi audio pipe closed")
