/* dump.c
	(C) 2010 Laurence Blaxter 

	For communicating with cc1020 based SDR
	streams a 8ksps wav file for use by fldigi
*/
#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <samplerate.h>/* For resampling support */
#include <stdint.h>  /* Integer support */
#include <stdlib.h>
#include <sys/select.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <setjmp.h>

typedef unsigned char  uint8;
typedef unsigned short uint16;
typedef void (*sighandler_t)(int);

int fd;
jmp_buf   Jump;

void broken_pipe(int sig)
{(void) signal(SIGPIPE, SIG_DFL);
longjmp(Jump,1);}

int initport(int fd) {
	struct termios options;
	// Get the current options for the port...
	tcgetattr(fd, &options);
	// Set the baud rates to 460800...
	cfsetispeed(&options, B460800);
	cfsetospeed(&options, B460800);
	// Enable the receiver and set local mode...
	options.c_cflag |= (CLOCAL | CREAD);
	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;
	options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
	options.c_oflag &= ~OPOST;
	options.c_cc[VMIN]  = 2;
	options.c_cc[VTIME] = 20;
	// Set the new options for the port...
	tcsetattr(fd, TCSANOW, &options);
	return 1;
}

int getbaud(int fd) {
	struct termios termAttr;
	int inputSpeed = -1;
	speed_t baudRate;
	tcgetattr(fd, &termAttr);
	/* Get the input speed.                              */
	baudRate = cfgetispeed(&termAttr);
	switch (baudRate) {
		case B0:      inputSpeed = 0; break;
		case B50:     inputSpeed = 50; break;
		case B110:    inputSpeed = 110; break;
		case B134:    inputSpeed = 134; break;
		case B150:    inputSpeed = 150; break;
		case B200:    inputSpeed = 200; break;
		case B300:    inputSpeed = 300; break;
		case B600:    inputSpeed = 600; break;
		case B1200:   inputSpeed = 1200; break;
		case B1800:   inputSpeed = 1800; break;
		case B2400:   inputSpeed = 2400; break;
		case B4800:   inputSpeed = 4800; break;
		case B9600:   inputSpeed = 9600; break;
		case B19200:  inputSpeed = 19200; break;
		case B38400:  inputSpeed = 38400; break;
		case B57600:  inputSpeed = 57600; break;
		case B115200: inputSpeed = 115200; break;
		case B460800: inputSpeed = 460800; break;
	}
	return inputSpeed;
}

int kbhit()
{
    struct timeval tv = { 0L, 0L };
    fd_set fds;
    FD_SET(0, &fds);
    return select(1, &fds, NULL, NULL, &tv);
}

int getch()
{
    int r;
    unsigned char c;
    if ((r = read(0, &c, sizeof(c))) < 0) {
        return r;
    } else {
        return c;
    }
}

int main(int argc, char **argv)
{
	uint8_t sResult[2];
	//create a header for the largest possible wave file, 8ksps, mono.
	uint8_t header[44]={0x52,0x49,0x46,0x46,0xff,0xff,0xff,0xff,0x57,0x41,0x56,0x45,0x66,0x6D,0x74,0x20,0x10,0x00,0x00,0x00,
0x01,0x00,0x01,0x00,0x40,0x1F,0x00,0x00,0x80,0x3E,0x00,0x00,0x02,0x00,0x10,0x00,0x64,0x61,0x74,0x61,0xD3,0xFF,0xFF,0xFF};
	int i,j,error,n;
	FILE* streamfid;
	uint16_t dataout_;
	int8_t retune,character;
	float datain, dataout;
	SRC_STATE* new_src=src_new(SRC_SINC_BEST_QUALITY,1,&error);//open a new sample rate convertor
	SRC_DATA data;					//set up the data struct to use
	data.input_frames=(long)1;			//we only ever send a single frame at a time
	data.output_frames=(long)1;			//we only allow room for a single frame output (can only downsample)
	data.src_ratio=(float)8000/(float)19200;	//downsampling to 8ksps
	data.end_of_input=0;				//a constant stream
	data.data_in=&datain;				//setup the data in out pointers
	data.data_out=&dataout;
	(void) signal(SIGPIPE,broken_pipe);
	if( argc !=2 )					//program name and device location
	{
		printf("usage: ./dump /dev/tty<port>\n");
		goto die;
	}
	if(*src_new==NULL)
	{
		printf("Samplerate error:%s\n",src_strerror(error));//gives human readable output if there was an error
		goto die;
	}
	if ( unlink("stream.wav") != 0)			//remove stream.wav
	{
		perror("make_fifo: Unable to delete old stream.wav fifo- ");
		goto die;
	}
	if ( mkfifo("stream.wav",0644) != 0)		//use system to make the wav file fifo
	{
		perror("make_fifo: Unable to make stream.wav fifo- ");
		goto die;
	}
	printf("waiting for the stream to be opened\n");
	streamfid = fopen("stream.wav" , "w");		//open the stream for writing	
	if (streamfid < 0)
	{
		perror("open_stream: Unable to open stream.wav - ");
		goto die;
	}
	fwrite(header, 1, 44, streamfid);		//the header is 44 bytes long
	fd = open(argv[1], O_RDWR | O_NOCTTY);		//note uses command line argument
	if (fd < 0)
	{
		perror("open_port: Unable to open serial device - ");
		goto die;
	}
	else
	{
		fcntl(fd, F_SETFL, 0);
	}
	printf("old baud=%d\n", getbaud(fd));
	initport(fd);
	printf("new baud=%d\n", getbaud(fd));
	character='s';
	if (write(fd, &character,1)<0)
	{
		perror("write failed - ");
		close(fd);
		goto die;
	}
	usleep(10000);
	j=tcflush(fd, TCIFLUSH);			//flush buffer
	//i=read(fd,sResult,2);
	printf("input flushed - %d\n",j);
	j=0;
	while(1)					//this is our main loop
	{	
		if (kbhit())				//we got some data through
		{
			scanf("%d",&i);
			retune=(int8_t)i;		//typecast as a signed character
			character='t';
			if (write(fd, &character,1)<0)
			{
				perror("write t failed - ");
				close(fd);
				goto die;
			}
			usleep(10000);			//10ms pause to avoid overwriting uart register
			if (write(fd,&retune,1)<0)
			{
				perror("write integer failed - ");
				close(fd);
				goto die;
			}
			usleep(100000);			//100ms pause to allow the micro to restart with a new frequency
			character='s';
			if (write(fd, &character,1)<0)		//resyncronise with the datastream
			{
				perror("write integer failed - ");
				close(fd);
				goto die;
			}
			usleep(10000);
			tcflush(fd, TCIFLUSH);
			printf("input flushed\n");
		}
		for(n=0;n<5;n++)				//process ten incoming characters -  theres a sync every ten chars
		{
			if ((i=read(fd,sResult,2))<2)
			{
				printf("%d\n",i);
				perror("read failed - ");
				close(fd);
				goto die;
			}
			switch(j)				//run the local oscillator
			{
				case 0:
					i=128*sResult[0];
					break;
				case 1:
					i=-128*sResult[1];
					break;
				case 2:
					i=-128*sResult[0];
					break;
				case 3:
					i=128*sResult[1];
			}
			j+=1;
			if(j==4)
				j=0;
			datain=(float)i;			//the resampler works on floats
			if((error=src_process(new_src,&data)))	//run the resampler
			{
				printf("Samplerate error:%s\n",src_strerror(error));//gives human readable output if there was an error
				goto die;
			}
			if(data.output_frames_gen)		//we had some data back from the resampler
			{
				dataout_=(int16_t)dataout;	//chuck 16 bit signed integers into the wav stream
				fputc(dataout_&0x00FF,streamfid);//remember its little endian
				fputc(dataout_>>8,streamfid);
				fflush(streamfid);
			}
		}
		if ((i=read(fd,sResult,1))<1)
		{
			printf("%d\n",i);
			perror("read failed - ");
			close(fd);
			goto die;
		}
		while(sResult[0]!=(int8_t)127)			//loop until sync
		{
			if ((i=read(fd,sResult,1))<1)
			{
				printf("%d\n",i);
				perror("read failed - ");
				close(fd);
				goto die;
			}
		}
		if(setjmp(Jump))				//the signal arrived, we quit as fldigi died
			goto die;
	}
	die:
	close(fd);					//close everything before exiting
	fclose(streamfid);
	src_delete(new_src);
	return 0;
}


